#include <assert.h>
#include <stdlib.h>
#include <syslog.h>
#include <stdbool.h>
#include <stdint.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#define IN	0
#define OUT	1

#define LOW	0
#define HIGH	1

#define PIN	24
#define POUT	4

#define BUF_SIZE	3
#define DIRECTION_MAX	35
#define VALUE_MAX	30

static int gpio_export(int pin)
{
    char buffer[BUF_SIZE];
    ssize_t bytes_written;
    int fd;

    fd = open("/sys/class/gpio/export", O_WRONLY);
    if(-1 == fd) {
        fprintf(stderr, "Failed to open export for writing!\n");
        return -1;
    }

    bytes_written = snprintf(buffer, BUF_SIZE, "%d", pin);
    write(fd, buffer, bytes_written);
    close(fd);

    return 0;
}

static int gpio_unexport(int pin)
{
    char buffer[BUF_SIZE];
    ssize_t bytes_written;
    int fd;

    fd = open("/sys/class/gpio/unexport", O_WRONLY);
    if(-1 == fd) {
        fprintf(stderr, "Failed to open unexport for writing!\n");
        return -1;
    }

    bytes_written = snprintf(buffer, BUF_SIZE, "%d", pin);
    write(fd, buffer, bytes_written);
    close(fd);

    return 0;
}

static int gpio_direction(int pin, int dir)
{
    static const char directions_str[]  = "in\0out";
    char path[DIRECTION_MAX];
    int fd;

    snprintf(path, DIRECTION_MAX, "/sys/class/gpio/gpio%d/direction", pin);
    fd = open(path, O_WRONLY);
    if(-1 == fd) {
        fprintf(stderr, "Failed to open gpio direction for writing!\n");
        return -1;
    }

    if(-1 == write(fd, &directions_str[IN == dir ? 0 : 3], IN == dir ? 2 : 3)) {
        fprintf(stderr, "Failed to set direction!\n");
        return -1;
    }
    close(fd);

    return 0;
}

static int gpio_read(int pin)
{
    char path[VALUE_MAX];
    char value_str[BUF_SIZE];
    int fd;

    snprintf(path, VALUE_MAX, "/sys/class/gpio/gpio%d/value", pin);
    fd = open(path, O_RDONLY);
    if(-1 == fd) {
        fprintf(stderr, "Failed to open gpio value for reading!\n");
        return -1;
    }

    if(-1 == read(fd, value_str, BUF_SIZE)) {
        fprintf(stderr, "Failed to read value!\n");
        return -1;
    }
    close(fd);

    return(atoi(value_str));
}

static int gpio_write(int pin, int value)
{
    static const char s_values_str[] = "01";
    char path[VALUE_MAX];
    int fd;

    snprintf(path, VALUE_MAX, "/sys/class/gpio/gpio%d/value", pin);
    fd = open(path, O_WRONLY);
    if(-1 == fd) {
        fprintf(stderr, "Failed to open gpio value for writing!\n");
        return -1;
    }

    if(1 != write(fd, &s_values_str[LOW == value ? 0 : 1], 1)) {
        fprintf(stderr, "Failed to write value!\n");
        return -1;
    }
    close(fd);

    return 0;
}

int main(void)
{
    int repeat = 10;

    if(-1 == gpio_export(POUT) || -1 == gpio_export(PIN)) {
        return(1);
    }

    if(-1 == gpio_direction(POUT, OUT) || -1 == gpio_direction(PIN, IN)) {
        return(2);
    }

    do {
        if(-1 == gpio_write(POUT, repeat % 2)) {
            return(3);
        }

	printf("I'm reading %d in GPIO %d\n", gpio_read(PIN), PIN);
        usleep(500 * 1000);
    } while(repeat--);

    if(-1 == gpio_unexport(POUT) || -1 == gpio_unexport(PIN)) {
	return(4);
    }

    return(0);
}

